-- AutoBuffCancel
-- Cancels unwanted buffs when they are detected

local RealmName;
local PlayerName;
local ABC_Initialized;
local ABC_Orig_GameTooltip_ClearMoney;

------------------------------------------------------------------------------
-- Yay tooltip kludges (thanks go to Telo's code!)

local function ABC_MoneyToggle()
	if (ABC_Orig_GameTooltip_ClearMoney) then
		GameTooltip_ClearMoney = ABC_Orig_GameTooltip_ClearMoney;
		ABC_Orig_GameTooltip_ClearMoney = nil;
	else
		ABC_Orig_GameTooltip_ClearMoney = GameTooltip_ClearMoney;
		GameTooltip_ClearMoney = ABC_GameTooltip_ClearMoney;
	end
end

function ABC_GameTooltip_ClearMoney()
	-- GNDN
end

------------------------------------------------------------------------------

function ABC_OnLoad()
	ABC_Initialized = nil;
	ABC_InitHooks();

	RealmName = string.gsub(GetCVar("realmName"),"'","");
	PlayerName = UnitName("player");
end

function ABC_OnEvent()
	if (event == "VARIABLES_LOADED") then
		ABC_VerifyData();
	elseif (event == "PLAYER_AURAS_CHANGED") then
		ABC_ScanBuffs();
	end
end

------------------------------------------------------------------------------

function ABC_InitHooks()
	-- Register slash command
	SLASH_AUTOBC1 = "/autobc";
	SlashCmdList["AUTOBC"] = ABC_CommandHandler;

	-- Register events
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_AURAS_CHANGED");
end

function ABC_VerifyData()
	if (not ABCState) then
		ABCState = {};
	end
	if (not ABCState[RealmName]) then
		ABCState[RealmName] = {};
	end
	if (not ABCState[RealmName][PlayerName]) then
		ABCState[RealmName][PlayerName] = {};
	end
	if (not ABCState[RealmName][PlayerName].BuffList) then
		ABCState[RealmName][PlayerName].BuffList = {};
	end
	--if (not ABCState[RealmName][PlayerName].Enabled) then
	--	ABCState[RealmName][PlayerName].Enabled = 1;
	--end
	ABC_Initialized = 1;
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(ABC_LOADED);
	end
end

function ABC_ScanBuffs()
	local n;
	if not ABC_Initialized then return; end

	if (not ABCState[RealmName][PlayerName].Enabled) then
		return;
	end

	for i = 1, table.getn(ABCState[RealmName][PlayerName].BuffList) do
		n = ABC_CheckForBuff(ABCState[RealmName][PlayerName].BuffList[i])
		if (n and (n >= 0) and (n <= 15)) then
			CancelPlayerBuff(n);
			ABC_Message(string.format(ABCMSG_CANCELLED, ABCState[RealmName][PlayerName].BuffList[i]));
		end
	end
end

function ABC_CheckForBuff(BuffName)
	local i;
	local n;
	local name;
	-- FIXME: Hmm, should this really be hardwired?
	for i = 0, 15 do
		ABC_MoneyToggle();
		ABCTooltip:SetOwner(UIParent, "ANCHOR_NONE");
		ABCTooltip:SetPlayerBuff(i);
		ABC_MoneyToggle();
		n = getglobal("ABCTooltipTextLeft1"):GetText();
		if n then name = string.lower(n); else name = ""; end

		-- FIXME: This could be a regex you know...
		--ABC_Message("Found buff: "..name);
		if (name == BuffName) then
			return i;
		end
	end
	return -1;
end

function ABC_CommandHandler(args)
	if (not args) or (args == "") then
		ABC_ShowStatus();
		return;
	end
	local opts=string.lower(args);	
	local BuffName;

	if (opts == ABCSLASH_HELP) then
		ABC_ShowHelp();	
		return;
	elseif (opts == ABCSLASH_ENABLE) then
		ABCState[RealmName][PlayerName].Enabled = 1;
		ABC_Message(string.format(ABCSTAT_HEADER, ABCSTAT_ENABLED));
	elseif (opts == ABCSLASH_DISABLE) then
		ABCState[RealmName][PlayerName].Enabled = nil;
		ABC_Message(string.format(ABCSTAT_HEADER, ABCSTAT_DISABLED));
	elseif (string.find(opts, ABCSLASH_ADD.." ") == 1) then
		BuffName = string.sub(opts, string.len(ABCSLASH_ADD) + 2, string.len(opts));
		ABC_AddBuff(BuffName);
	elseif (string.find(opts, ABCSLASH_REMOVE.." ") == 1) then
		BuffName = string.sub(opts, string.len(ABCSLASH_REMOVE) + 2, string.len(opts));
		ABC_RemoveBuff(BuffName);
	else
		ABC_Error(string.format(ABCERROR_BADCMD, args));
	end
end

function ABC_AddBuff(BuffName)
	if (not BuffName) then return; end
	-- Note due to command line processing, buffname will be lowercase
	local i;
	for i = 1, table.getn(ABCState[RealmName][PlayerName].BuffList) do
		if (BuffName == ABCState[RealmName][PlayerName].BuffList[i]) then
			ABC_Error(string.format(ABCERROR_BUFFEXISTS, BuffName));
			return;
		end
	end
	table.insert(ABCState[RealmName][PlayerName].BuffList, BuffName);
	ABC_Message(string.format(ABCMSG_ADD, BuffName));
end

function ABC_RemoveBuff(BuffName)
	if (not BuffName) then return; end
	-- Note: FIXME: BuffName could be a number...
	local i;
	for i = 1, table.getn(ABCState[RealmName][PlayerName].BuffList) do
		if (BuffName == ABCState[RealmName][PlayerName].BuffList[i]) then
			table.remove(ABCState[RealmName][PlayerName].BuffList, i);
			ABC_Message(string.format(ABCMSG_REMOVE, BuffName));
			return;
		end
	end
	ABC_Error(string.format(ABCERROR_BUFFNOTFOUND, BuffName));
end

function ABC_Message(msg)
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

function ABC_Error(msg)
	ABC_Message(ABCERROR_HEADER..msg);
	ABC_Message(ABCSTAT_HELPLINE);
end

------------------------------------------------------------------------------
-- Status and help

function ABC_ShowStatus()
	local statstring;
	local i;

	if ABCState[RealmName][PlayerName].Enabled then
		statstring = ABCSTAT_ENABLED;
	else
		statstring = ABCSTAT_DISABLED;
	end

	ABC_Message(ABC_COLOUR..ABC_HEADER);
	ABC_Message(string.format(ABCSTAT_HEADER, statstring));
	ABC_Message(ABCSTAT_LISTHEADER);
	for i = 1, table.getn(ABCState[RealmName][PlayerName].BuffList) do
		ABC_Message(string.format("     %d - %s", i, ABCState[RealmName][PlayerName].BuffList[i]));
	end
end

function ABC_ShowHelp()
	for i = 1, table.getn(ABC_HELPTEXT) do
		ABC_Message(ABC_HELPTEXT[i]);
	end
end

